function [testmean, trainmean] = CV(model,K)
% A K-fold cross validation function to test
% the performance of a model for a selected database
%
% >> testmean = CV(model,K)
% >> [testmean, trainmean] = CV(model,K)
%
% Input
% 'model' is a structure which includes the fields 'func','X','Y','delta','gam','mu'
% 'K' number of folds: is a scalar
%
% Output
% testmean the mean of the c-index on K-fold test:  is a scalar
% trainmean the mean of the c-index on K-fold test:  is a scalar
%
% Author: Liu,Yang (liu.yang@it.uu.se)


% Set the size of trainning and test sets
n=size(model.X,1);
ntrain=ceil(n/K*(K-1));
ntest=n-ntrain;

ctest=zeros(K,1);
ctrain=zeros(K,1);
for i= 1:K
    % Cut the model
    indtrain=[1:(i-1)*ntest i*ntest+1:n];
    indtest=(i-1)*ntest+1:i*ntest;
    [trainmodel,testmodel]=seperatemodel(model,indtrain,indtest);
    % Solve it
    w=feval(trainmodel.func,trainmodel);
    ctest(i)=cindex(testmodel.X*w,testmodel.Y,testmodel.delta);
    ctrain(i)=cindex(trainmodel.X*w,trainmodel.Y,trainmodel.delta);
end

% calculate the average of c-index
testmean=mean(ctest);
trainmean=mean(ctrain);