function [w, l] = coxph(model,Y,delta)
% Implementation of minlip using Yalmip
% Use censored failure time data
% event(not censored): delta=1;  censored: delta=0
%
% >> w=minlip(model)
% >> w=minlip(X,Y,delta,gam)
% >> [w,ep]=minlip(model)
% >> [w,ep]=minlip(X,Y,delta,gam)
%
% Input:
% Case 1: Structure format:
% 'model' is a structure which includes the fields 'func','X','Y','delta','gam','mu'
% Case 2: Original format:
% 'X' input: is a matrix of n x d
% 'Y' time: is a vector of n elements
% 'delta' censoring: is a vector of n elements (0-1 values, 0 means events and 1 means censored)
% 'gam' a parameter: is a scalar
%
% Output:
% 'w' parameter: is a vector of d elements
% 'ep' residuals: is a vector of n elements
%
% Author: Liu,Yang (liu.yang@it.uu.se)

% Selection of input format
if nargin==1
    [~,X,Y,delta]=ungroup(model);
else
    X=model;
end

[n,d]=size(X);

% Sort by Y
[Y,events] = sort(Y);
X= X(events,:);
delta=delta(events,:);
events = find(delta);
ne     = length(events);

% Build matrix D and Dc
%Dc=buildDMatrix(n,delta,Y);
%Dc=MakeDiff2(Y,1-delta);

% Solve QP by solver
yalmip('clear');
w=sdpvar(d,1);
l=sdpvar(ne,1);
C=[];
for i=1:ne
    C=[C,l(i)>= -X(events(i),:)*w+logsumexp(X(events(i):n,:)*w)];
end
solvesdp(C,sum(l),sdpsettings('solver','fmincon','verbose',1,'cachesolvers',1));

% Convert the result to double type
w=double(w);
l=double(l);