function [x,xtest,y,ytest,deltatrain,deltatest]=cut(model,indtrain,indtest,perm)
% A cut function to cut the model data to two parts based on a permute
%
% >> [trainmodel,testmodel]=cut(model,indtrain,indtest)
% >> [trainmodel,testmodel]=cut(model,indtrain,indtest,perm)
% >> [x,xtest,y,ytest,deltatrain,deltatest]=cut(model,indtrain,indtest)
% >> [x,xtest,y,ytest,deltatrain,deltatest]=cut(model,indtrain,indtest,perm)
%
% Input
% model is a structure includes fields func,X,Y,delta,gam,mu
%   X input: is a matrix of n x d
%   Y time: is a vector of n elements
%   delta censoring: is a vector of n elements (0-1 values, 0 means events and 1 means censored)
%   func the function name of model we want to test: is a string
%   gam a parameter: is a scalar
%   mu a parameter: is a scalar
% indtrain is a n1 vector indicates the index of the training set
% indtest is a n2 vector indicates the index of the test set
% perm is a n vector indicates the random permute
%
% Output
% CASE 1: Grouped data
% trainmodel the training set: is a structure as model
% testmodel the test set: is a structure as model
%
% CASE 2: Ungrouped data
% x input of train set: is a matrix of n1 x d
% xtest input of test set: is a matrix of n2 x d
% y output of train set: is a vector of n1 elements
% ytest output of test set: is a vector of n2 elements
% delta censoring of train set: is a vector of n1 elements
% delta censoring of test set: is a vector of n2 elements



if nargin>3
    indtrain=perm(indtrain);
    indtest=perm(indtest);
end

[func,X,Y,delta,gam,miu]=unfold(model);

deltatrain=delta(indtrain);
deltatest=delta(indtest);
x=X(indtrain,:);
y=Y(indtrain);
xtest=X(indtest,:);
ytest=Y(indtest);

if nargout<6
    x=buildmodel(func,x,y,deltatrain,gam,miu);
    xtest=buildmodel(func,xtest,ytest,deltatest,gam,miu);
end
    