clear;
fprintf('\n');
% numcores=4;
% if numcores>1
%     if matlabpool('size')>0
%         matlabpool close;
%     end
%     eval(['matlabpool local ' num2str(numcores)]);    
% end

disp('--------------------------------START--------------------------------');
disp('This demo shows how we can use this program.');
disp('We can use the different search methods to test a specified model with a selected dataset.');
disp('When the program runs under multi-core environment it will turn to paralell mode automatically');

dataset='sorliedataorig'; % Dataset
func='minlip_kernelly'; % Model function
tunemethod='linesearch'; % Tune method
tunepar={-25,10,1,0.5}; % Tune parameters (serach range/initial values)
testmethod='CVk'; % Test method
testpar={3}; % Test parameters
nr=10;

% Start
tic

% Load the selected database
load(dataset);
model=buildmodel(func,X,Y,delta);

n=size(X,1);
ntrain=ceil(n/3*2);
ntest=n-ntrain;
indtrain=ntest+1:n;
indtest=1:ntest;

parfor i=1:nr
% Seperate it into train set and test set
[trainmodel,testmodel]=seperatemodel(model,indtrain,indtest,randperm(n));

% Change the signs if needed
if strcmp(func,'model2p') || strcmp(func,'minlipp')
    [trainmodel.X, testmodel.X]=changesign(trainmodel,testmodel.X);
end

trainmodel2=trainmodel;
trainmodel2.func='minlip';
testmodel2=testmodel;
trainmodel2.gam=tune(trainmodel2,tunemethod,tunepar,'CV',testpar);
ctest2(i)=cindex(testmodel2.X*feval('minlip',trainmodel2),testmodel2.Y,testmodel2.delta);

% Tune the trainmodel
bestpars=tune(trainmodel,tunemethod,tunepar,testmethod,testpar);
gams(i)=bestpars(1);
trainmodel.gam=bestpars(1);
if length(bestpars)>1
    mus(i)=bestpars(2);
    trainmodel.mu=bestpars(2);
end
% Get w based on training set using the tuned parameters
trainmodel.kernel='lin_kernel';
trainmodel.par=[];
trainmodel.Xt=testmodel.X;
[a,u]=feval(func,trainmodel);
ctest(i)=cindex(u,testmodel.Y,testmodel.delta);
lrtest(i)=getlogrank(testmodel.Y,testmodel.delta,u);
hrtest(i)=gethr(testmodel.Y,testmodel.delta,-u);

% w=feval(func,trainmodel);
% 
% %Calculate the c-index on both sets
% ctest(i)=cindex(testmodel.X*w,testmodel.Y,testmodel.delta);
% ctrain(i)=cindex(trainmodel.X*w,trainmodel.Y,trainmodel.delta);
% lrtest(i)=getlogrank(testmodel.Y,testmodel.delta,testmodel.X*w);
% lrtrain(i)=getlogrank(trainmodel.Y,trainmodel.delta,trainmodel.X*w);
% hrtest(i)=gethr(testmodel.Y,testmodel.delta,-testmodel.X*w);
% hrtrain(i)=gethr(trainmodel.Y,trainmodel.delta,-trainmodel.X*w);

fprintf('...%s\n',num2str(i));
end
disp('Time cost:')
toc

ctestmedian=median(ctest)
lrtestmedian=median(lrtest)
hrtestmedian=median(hrtest)

figure;
boxplot([ctest']);

disp('---------------------------------END---------------------------------');
% if matlabpool('size')>0  
%     matlabpool close;
% end
fprintf('\n');