function [bestgam,bestmu] = gridsearch2(model,par,method,methodpar)
% A tune function to gridsearch2 the best parameters
%
% >> [bestgam,bestmu] = gridsearch2(model,par,method,methodpar)
%
% Input
% 'model' is a structure which includes the fields 'func','X','Y','delta','gam','mu'
% 'par' is the parameters setting of linesearch ({left,right,interval length,threshold})
% 'method' is a string of the name of the test testmethod we use ('CV','CV2')
% 'methodpar' is the parameters setting of the test testmethod ({K})
%
% Output
% 'bestgam' the best gam parameter: is a scalar
% 'bestmu' the best mu parameter: is a scalar
%
% Author: Liu,Yang (liu.yang@it.uu.se)

% Preprocess
[l,r,s,threshold]=par{:};
l2=l; r2=r;
len=r-l;
len2=len;
d=ceil(1/s);%floor(sqrt(len))-
s2=s;

% Loop until the s is small enough (small than the threshold)

%C = progress('init',['Determine initial tuning parameters...',': # searching cycle(s) ', num2str(k)]);
% Set index and gams
index1=l:s:r;
index2=l2:s2:r2;

% init c
c=zeros(length(index1),length(index2));

for j=1 :length(index1)
    model.gam=exp(index1(j));
    for m=1:length(index2)
        model.mu=exp(index2(m));
        c(j,m)=feval(method,model,methodpar{:});
    end
    %C = progress(C,j/length(index1));
end

[bestc,ii]=max(c(:));
[i,i2]=ind2sub(size(c),ii);
c(i,i2)=-inf;
[bestc2,ii]=max(c(:));
[i3,i4]=ind2sub(size(c),ii);

C1=index1(i);
C2=index2(i2);

% Update the left and right boundary
l=index1(i)-3*s/4;
l2=index2(i2)-3*s2/4;
r=index1(i)+3*s/4;
r2=index2(i2)+3*s2/4;
l3=index1(i3)-3*s/4;
l4=index2(i4)-3*s2/4;
r3=index1(i3)+3*s/4;
r4=index2(i4)+3*s2/4;

range1=l:s/4:r;
range2=l2:s2/4:r2;
range3=l3:s/4:r3;
range4=l4:s2/4:r4;

rangeA=[range1,range3];
if i3==i+1 rangeA=l:s/4:r3;end
if i3==i-1 rangeA=l3:s/4:r;end
rangeB=[range2,range4];
if i4==i2+1 rangeB=l2:s2/4:r4;end
if i4==i2-1 rangeB=l4:s2/4:r2;end

for j=1 :length(rangeA)
    model.gam=exp(rangeA(j));
    for m=1:length(rangeB)
        model.mu=exp(rangeB(m));
        c(j,m)=feval(method,model,methodpar{:});
    end
end

[bestccc,ii]=max(c(:));
if bestccc>bestc
    [i,i2]=ind2sub(size(c),ii);
    C1=rangeA(i);
    C2=rangeB(i2);
end

bestgam=exp(C1);
bestmu=exp(C2);
