function [bestgam,bestpar] = gridsearchk(model,par,method,methodpar)
% A tune function to gridsearch the best parameters
%
% >> [bestgam,bestmu] = gridsearch(model,par,method,methodpar)
%
% Input
% 'model' is a structure which includes the fields 'func','X','Y','delta','gam','mu'
% 'par' is the parameters setting of linesearch ({left,right,interval length,threshold})
% 'method' is a string of the name of the test testmethod we use ('CV','CV2')
% 'methodpar' is the parameters setting of the test testmethod ({K})
%
% Output
% 'bestgam' the best gam parameter: is a scalar
% 'bestmu' the best mu parameter: is a scalar
%
% Author: Liu,Yang (liu.yang@it.uu.se)

% Preprocess
[l,r,s,threshold,l2,r2]=par{:};
len=r-l;
len2=len;
d=1;%floor(sqrt(len))-
s2=1;

% Loop until the s is small enough (small than the threshold)
k=0;
while (s>threshold && s2>threshold)
    k=k+1;
    %C = progress('init',['Determine initial tuning parameters...',': # searching cycle(s) ', num2str(k)]);
    % Set index and gams
    index1=l:s:r;
    index2=l2:s2:r2;
    
    % init c
    c=zeros(length(index1),length(index2));
    
    for j=1 :length(index1)
        model.gam=exp(index1(j));
        for m=1:length(index2)
            model.par=[exp(index2(m))];
            c(j,m)=feval(method,model,methodpar{:});
        end
        %C = progress(C,j/length(index1));
    end
    
    [bestc,ii]=max(c(:));
    [i,i2]=ind2sub(size(c),ii);    
    
    % Update the left and right boundary
    l=index1(i)-d*s;
    l2=index2(i2)-d*s2;
    r=index1(i)+d*s;
    r2=index2(i2)+d*s2;
    
    s=(r-l)/len*2;
    s2=(r2-l2)/len2*2;
    
%     filename=strcat('gridsearchC',num2str(k));
%     save(filename,'index1','index2','c');
end
% for i=1:k
%     filename=strcat('gridsearchC',num2str(i));
%     load(filename);
%     figure;
%     surfc(index1,index2,c);
%     colorbar;
%     xlabel('log(gam)');
%     ylabel('log(mu)');
%     zlabel('c-index');
%     title('C-index by defferent parameters');
% end

% return best gammar
C1=index1(i);
C2=index2(i2);
bestgam=exp(C1);
bestpar=exp(C2);
